# Licensed to Elasticsearch B.V. under one or more contributor
# license agreements. See the NOTICE file distributed with
# this work for additional information regarding copyright
# ownership. Elasticsearch B.V. licenses this file to you under
# the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

# frozen_string_literal: true

require 'hoss/context/request'
require 'hoss/context/request/socket'
require 'hoss/context/request/url'
require 'hoss/context/response'
require 'hoss/context/user'

module Hoss
  # @api private
  class Context
    def initialize(custom: {}, labels: {}, user: nil, service: nil)
      @custom = custom
      @labels = labels
      @user = user || User.new
      @service = service
    end

    Service = Struct.new(:framework)
    Framework = Struct.new(:name, :version)

    attr_accessor :request
    attr_accessor :response
    attr_accessor :user
    attr_reader :custom
    attr_reader :labels
    attr_reader :service

    # rubocop:disable Metrics/CyclomaticComplexity
    def empty?
      return false if labels.any?
      return false if custom.any?
      return false if user.any?
      return false if service
      return false if request || response

      true
    end
    # rubocop:enable Metrics/CyclomaticComplexity

    def set_service(framework_name: nil, framework_version: nil)
      @service = Service.new(
        Framework.new(framework_name, framework_version)
      )
    end
  end
end
