# Licensed to Elasticsearch B.V. under one or more contributor
# license agreements. See the NOTICE file distributed with
# this work for additional information regarding copyright
# ownership. Elasticsearch B.V. licenses this file to you under
# the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

# frozen_string_literal: true

module Hoss
  module Transport
    module Serializers
      # @api private
      class EventSerializer < Serializer
        def build(event)
          r = {
              eventId: event.id,
              request: build_request(event.request),
          }
          r['response'] = build_response(event.response) if event.response
          r['error'] = build_error(event.error) if event.error
          r
        end

        private

        def build_request(request)
          return unless request
          body = request.body ? Base64.encode64(request.body).strip : nil

          {
            method: request.method,
            url: request.url,
            headers: request.headers,
            body: body || '',
            receivedAt: request.received_at
          }
        end

        def build_response(response)
          return unless response
          body = response.body ? Base64.encode64(response.body).strip : nil

          {
            statusCode: response.status_code,
            headers: response.headers,
            body: body || '',
            receivedAt: response.received_at
          }
        end

        def build_error(error)
          return unless error

          {
            type: error.type,
            receivedAt: error.received_at
          }
        end
      end
    end
  end
end
