# Licensed to Elasticsearch B.V. under one or more contributor
# license agreements. See the NOTICE file distributed with
# this work for additional information regarding copyright
# ownership. Elasticsearch B.V. licenses this file to you under
# the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

# frozen_string_literal: true

module Hoss
  module Transport
    # @api private
    class UserAgent
      def initialize(config)
        @built = build(config)
      end

      def to_s
        @built
      end

      private

      def build(config)
        metadata = Metadata.new(config)

        [
          "hoss-ruby/#{VERSION}",
          HTTP::Request::USER_AGENT,
          [
            metadata.service.runtime.name,
            metadata.service.runtime.version
          ].join('/')
        ].join(' ')
      end
    end
  end
end
